/*
 * AllTray.vala - Entrypoint for AllTray.
 * Copyright (c) 2009 Michael B. Trausch <mike@trausch.us>
 */

#include "AllTray.h"
#include <libwnck/libwnck.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <config.h>
#include <signal.h>
#include "Debug.h"
#include "Process.h"




static gboolean string_contains (const char* self, const char* needle);
struct _AllTrayProgramPrivate {
	char** _args;
	gint _args_length1;
	gint _args_size;
	char** _cleanArgs;
	gint _cleanArgs_length1;
	gint _cleanArgs_size;
	GList* _plist;
};

#define ALL_TRAY_PROGRAM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ALL_TRAY_TYPE_PROGRAM, AllTrayProgramPrivate))
enum  {
	ALL_TRAY_PROGRAM_DUMMY_PROPERTY,
	ALL_TRAY_PROGRAM_ARGS
};
static void _g_list_free_g_object_unref (GList* self);
static gboolean all_tray_program__cl_debug = FALSE;
static gboolean all_tray_program__display_ver = FALSE;
static AllTrayProgram* all_tray_program__instance = NULL;
static WnckScreen* all_tray_program__wnckScreen = NULL;
static char** _vala_array_dup1 (char** self, int length);
static char** _vala_array_dup2 (char** self, int length);
static void all_tray_program_display_version (AllTrayProgram* self);
static void _all_tray_program_cleanup_for_process_all_tray_process_process_died (AllTrayProcess* _sender, AllTrayProcess* p, gpointer self);
static void all_tray_program_spawn_new_process (AllTrayProgram* self);
static void _all_tray_program_sighandler_native_std_csignal_signal_handler (gint whichSignal);
static void all_tray_program_install_signal_handlers (AllTrayProgram* self);
static void all_tray_program_cleanup_for_process (AllTrayProgram* self, AllTrayProcess* p);
static char** all_tray_program_get_command_line (AllTrayProgram* self, char** args, int args_length1, int* result_length1);
static void all_tray_program_sighandler (gint caught_signal);
static gpointer all_tray_program_parent_class = NULL;
static void all_tray_program_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GOptionEntry ALL_TRAY_PROGRAM__acceptedCmdLineOptions[] = {{"debug", 'D', 0, G_OPTION_ARG_NONE, &all_tray_program__cl_debug, "Enable debugging messages", NULL}, {"version", 'v', 0, G_OPTION_ARG_NONE, &all_tray_program__display_ver, "Display AllTray version info", NULL}, {NULL}};


#line 929 "glib-2.0.vapi"
static gboolean string_contains (const char* self, const char* needle) {
#line 929 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 929 "glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 930 "glib-2.0.vapi"
	return strstr (self, needle) != NULL;
}


GQuark all_tray_all_tray_error_quark (void) {
	return g_quark_from_static_string ("all_tray_all_tray_error-quark");
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


static char** _vala_array_dup1 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp0;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp0 = self[i], (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	}
	return result;
}


#line 37 "AllTray.vala"
AllTrayProgram* all_tray_program_construct (GType object_type, char*** args, int* args_length1) {
	AllTrayProgram * self;
	char** _tmp2;
	char** _tmp1;
	const char* _tmp0;
	char** _tmp4;
	gint _tmp3;
	self = g_object_newv (object_type, 0, NULL);
	_tmp2 = NULL;
#line 38 "AllTray.vala"
	_tmp1 = NULL;
#line 14 "AllTray.vala"
	_tmp0 = NULL;
#line 38 "AllTray.vala"
	self->priv->_args = (_tmp2 = (_tmp1 = (*args), (_tmp1 == NULL) ? ((gpointer) _tmp1) : _vala_array_dup1 (_tmp1, *args_length1)), self->priv->_args = (_vala_array_free (self->priv->_args, self->priv->_args_length1, (GDestroyNotify) g_free), NULL), self->priv->_args_length1 = *args_length1, self->priv->_args_size = self->priv->_args_length1, _tmp2);
#line 39 "AllTray.vala"
	gdk_init (&self->priv->_args_length1, &self->priv->_args);
	_tmp4 = NULL;
#line 40 "AllTray.vala"
	self->priv->_cleanArgs = (_tmp4 = all_tray_program_command_line_init (self, &self->priv->_args, &self->priv->_args_length1, &_tmp3), self->priv->_cleanArgs = (_vala_array_free (self->priv->_cleanArgs, self->priv->_cleanArgs_length1, (GDestroyNotify) g_free), NULL), self->priv->_cleanArgs_length1 = _tmp3, self->priv->_cleanArgs_size = self->priv->_cleanArgs_length1, _tmp4);
#line 42 "AllTray.vala"
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine, ALL_TRAY_DEBUG_LEVEL_Information, "Command line options parsed.");
	return self;
}


#line 37 "AllTray.vala"
AllTrayProgram* all_tray_program_new (char*** args, int* args_length1) {
#line 37 "AllTray.vala"
	return all_tray_program_construct (ALL_TRAY_TYPE_PROGRAM, args, args_length1);
}


static char** _vala_array_dup2 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp9;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp9 = self[i], (_tmp9 == NULL) ? NULL : g_strdup (_tmp9));
	}
	return result;
}


#line 47 "AllTray.vala"
char** all_tray_program_command_line_init (AllTrayProgram* self, char*** args, int* args_length1, int* result_length1) {
	GError * inner_error;
	const char* _tmp0;
	char* default_flags;
	char** _tmp1;
	gint env_args_size;
	gint env_args_length1;
	char** env_args;
	GOptionContext* opt_ctx;
	char** _tmp10;
	const char* _tmp9;
	char** _tmp11;
	char** _tmp12;
#line 47 "AllTray.vala"
	g_return_val_if_fail (self != NULL, NULL);
	inner_error = NULL;
#line 48 "AllTray.vala"
	gtk_init (&(*args_length1), &(*args));
	/* First, see if there are environment arguments.*/
#line 52 "AllTray.vala"
	_tmp0 = NULL;
	default_flags = (_tmp0 = g_getenv ("ALLTRAY_OPTIONS"), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
#line 53 "AllTray.vala"
	_tmp1 = NULL;
	env_args = (_tmp1 = NULL, env_args_length1 = 0, env_args_size = env_args_length1, _tmp1);
#line 55 "AllTray.vala"
	if (default_flags != NULL) {
		GString* sb;
		char** _tmp3;
		char** _tmp2;
		sb = g_string_new ("");
#line 57 "AllTray.vala"
		g_string_append_printf (sb, "%s", (*args)[0]);
#line 58 "AllTray.vala"
		g_string_append_printf (sb, " %s", default_flags);
		_tmp3 = NULL;
		_tmp2 = NULL;
#line 59 "AllTray.vala"
		env_args = (_tmp3 = _tmp2 = g_strsplit (sb->str, " ", 0), env_args = (_vala_array_free (env_args, env_args_length1, (GDestroyNotify) g_free), NULL), env_args_length1 = _vala_array_length (_tmp2), env_args_size = env_args_length1, _tmp3);
		(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	}
	opt_ctx = g_option_context_new ("- Dock software to the systray");
#line 64 "AllTray.vala"
	g_option_context_add_main_entries (opt_ctx, ALL_TRAY_PROGRAM__acceptedCmdLineOptions, NULL);
#line 65 "AllTray.vala"
	g_option_context_add_group (opt_ctx, gtk_get_option_group (TRUE));
#line 67 "AllTray.vala"
	if (env_args != NULL) {
		{
#line 69 "AllTray.vala"
			g_option_context_parse (opt_ctx, &env_args_length1, &env_args, &inner_error);
			if (inner_error != NULL) {
				if (inner_error->domain == G_OPTION_ERROR) {
					goto __catch0_g_option_error;
				}
				goto __finally0;
			}
		}
		goto __finally0;
		__catch0_g_option_error:
		{
			GError * e;
			e = inner_error;
			inner_error = NULL;
			{
#line 71 "AllTray.vala"
				fprintf (stderr, "error: env flag parsing failed (%s)\n", e->message);
#line 73 "AllTray.vala"
				exit (1);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally0:
		if (inner_error != NULL) {
			default_flags = (g_free (default_flags), NULL);
			env_args = (_vala_array_free (env_args, env_args_length1, (GDestroyNotify) g_free), NULL);
			(opt_ctx == NULL) ? NULL : (opt_ctx = (g_option_context_free (opt_ctx), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return NULL;
		}
	}
	{
#line 78 "AllTray.vala"
		g_option_context_parse (opt_ctx, &(*args_length1), &(*args), &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_OPTION_ERROR) {
				goto __catch1_g_option_error;
			}
			goto __finally1;
		}
	}
	goto __finally1;
	__catch1_g_option_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
#line 80 "AllTray.vala"
			fprintf (stderr, "error: command line parsing failed (%s)\n", e->message);
#line 82 "AllTray.vala"
			exit (1);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (inner_error != NULL) {
		default_flags = (g_free (default_flags), NULL);
		env_args = (_vala_array_free (env_args, env_args_length1, (GDestroyNotify) g_free), NULL);
		(opt_ctx == NULL) ? NULL : (opt_ctx = (g_option_context_free (opt_ctx), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
#line 85 "AllTray.vala"
	if (all_tray_program__display_ver) {
#line 86 "AllTray.vala"
		all_tray_program_display_version (self);
#line 87 "AllTray.vala"
		exit (0);
	}
#line 89 "AllTray.vala"
	if (all_tray_program__cl_debug) {
#line 89 "AllTray.vala"
		all_tray_debug_notification_init ();
	}
	/* Strip "--" out of the command line arguments*/
#line 92 "AllTray.vala"
	if (_vala_strcmp0 ((*args)[1], "--") == 0) {
		GString* sb;
		char** _tmp6;
		gint newargs_size;
		gint newargs_length1;
		char** _tmp5;
		char** newargs;
		char** _tmp7;
		char** _tmp8;
		sb = g_string_new ("");
		{
			char** arg_collection;
			int arg_collection_length1;
			int arg_it;
#line 95 "AllTray.vala"
			arg_collection = (*args);
			arg_collection_length1 = *args_length1;
			for (arg_it = 0; arg_it < (*args_length1); arg_it = arg_it + 1) {
				const char* _tmp4;
				char* arg;
#line 47 "AllTray.vala"
				_tmp4 = NULL;
				arg = (_tmp4 = arg_collection[arg_it], (_tmp4 == NULL) ? NULL : g_strdup (_tmp4));
				{
#line 96 "AllTray.vala"
					if (_vala_strcmp0 (arg, "--") == 0) {
						arg = (g_free (arg), NULL);
#line 96 "AllTray.vala"
						continue;
					}
#line 97 "AllTray.vala"
					g_string_append_printf (sb, "%s@&@", arg);
					arg = (g_free (arg), NULL);
				}
			}
		}
#line 100 "AllTray.vala"
		_tmp6 = NULL;
		_tmp5 = NULL;
		newargs = (_tmp6 = _tmp5 = g_strsplit (sb->str, "@&@", 0), newargs_length1 = _vala_array_length (_tmp5), newargs_size = newargs_length1, _tmp6);
#line 101 "AllTray.vala"
		_tmp7 = NULL;
#line 101 "AllTray.vala"
		_tmp8 = NULL;
#line 101 "AllTray.vala"
		return (_tmp8 = (_tmp7 = newargs, *result_length1 = newargs_length1, _tmp7), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), default_flags = (g_free (default_flags), NULL), env_args = (_vala_array_free (env_args, env_args_length1, (GDestroyNotify) g_free), NULL), (opt_ctx == NULL) ? NULL : (opt_ctx = (g_option_context_free (opt_ctx), NULL)), _tmp8);
	}
#line 104 "AllTray.vala"
	_tmp10 = NULL;
#line 47 "AllTray.vala"
	_tmp9 = NULL;
#line 104 "AllTray.vala"
	_tmp11 = NULL;
#line 104 "AllTray.vala"
	_tmp12 = NULL;
#line 104 "AllTray.vala"
	return (_tmp12 = (_tmp11 = (_tmp10 = (*args), (_tmp10 == NULL) ? ((gpointer) _tmp10) : _vala_array_dup2 (_tmp10, *args_length1)), *result_length1 = *args_length1, _tmp11), default_flags = (g_free (default_flags), NULL), env_args = (_vala_array_free (env_args, env_args_length1, (GDestroyNotify) g_free), NULL), (opt_ctx == NULL) ? NULL : (opt_ctx = (g_option_context_free (opt_ctx), NULL)), _tmp12);
}


#line 107 "AllTray.vala"
gint all_tray_program_run (AllTrayProgram* self) {
	GError * inner_error;
	GList* _tmp0;
	WnckScreen* _tmp3;
	WnckScreen* _tmp2;
#line 107 "AllTray.vala"
	g_return_val_if_fail (self != NULL, 0);
	inner_error = NULL;
	_tmp0 = NULL;
#line 108 "AllTray.vala"
	self->priv->_plist = (_tmp0 = NULL, (self->priv->_plist == NULL) ? NULL : (self->priv->_plist = (_g_list_free_g_object_unref (self->priv->_plist), NULL)), _tmp0);
#line 110 "AllTray.vala"
	wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
#line 111 "AllTray.vala"
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Misc, ALL_TRAY_DEBUG_LEVEL_Information, "Set WNCK Client Type to PAGER");
#line 114 "AllTray.vala"
	all_tray_program_install_signal_handlers (self);
	{
#line 117 "AllTray.vala"
		all_tray_program_spawn_new_process (self);
	}
	goto __finally2;
	__catch2_all_tray_all_tray_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			gint _tmp1;
#line 119 "AllTray.vala"
			fprintf (stderr, e->message);
#line 120 "AllTray.vala"
			return (_tmp1 = 1, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), _tmp1);
		}
	}
	__finally2:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return 0;
	}
	_tmp3 = NULL;
#line 123 "AllTray.vala"
	_tmp2 = NULL;
#line 123 "AllTray.vala"
	all_tray_program__wnckScreen = (_tmp3 = (_tmp2 = wnck_screen_get_default (), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (all_tray_program__wnckScreen == NULL) ? NULL : (all_tray_program__wnckScreen = (g_object_unref (all_tray_program__wnckScreen), NULL)), _tmp3);
#line 125 "AllTray.vala"
	gtk_main ();
#line 126 "AllTray.vala"
	return 0;
}


#line 129 "AllTray.vala"
static void all_tray_program_display_version (AllTrayProgram* self) {
#line 129 "AllTray.vala"
	g_return_if_fail (self != NULL);
#line 130 "AllTray.vala"
	fprintf (stdout, "AllTray %s\n", PACKAGE_VERSION);
}


#line 167 "AllTray.vala"
static void _all_tray_program_cleanup_for_process_all_tray_process_process_died (AllTrayProcess* _sender, AllTrayProcess* p, gpointer self) {
	all_tray_program_cleanup_for_process (self, _sender);
}


#line 133 "AllTray.vala"
static void all_tray_program_spawn_new_process (AllTrayProgram* self) {
	GError * inner_error;
	char** _tmp1;
	gint a_size;
	gint a_length1;
	gint _tmp0;
	char** a;
	AllTrayProcess* p;
	AllTrayProcess* _tmp2;
#line 133 "AllTray.vala"
	g_return_if_fail (self != NULL);
	inner_error = NULL;
#line 134 "AllTray.vala"
	_tmp1 = NULL;
	a = (_tmp1 = all_tray_program_get_command_line (self, self->priv->_cleanArgs, self->priv->_cleanArgs_length1, &_tmp0), a_length1 = _tmp0, a_size = a_length1, _tmp1);
	p = all_tray_process_new (a, a_length1);
#line 136 "AllTray.vala"
	g_signal_connect_object (p, "process-died", (GCallback) _all_tray_program_cleanup_for_process_all_tray_process_process_died, self, 0);
#line 137 "AllTray.vala"
	all_tray_process_run (p);
#line 139 "AllTray.vala"
	if (!all_tray_process_get_running (p)) {
#line 140 "AllTray.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Error, "Whoops.  Not running?");
		inner_error = g_error_new_literal (ALL_TRAY_ALL_TRAY_ERROR, ALL_TRAY_ALL_TRAY_ERROR_FAILED, "Failed to start process");
		if (inner_error != NULL) {
			a = (_vala_array_free (a, a_length1, (GDestroyNotify) g_free), NULL);
			(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return;
		}
	}
#line 146 "AllTray.vala"
	_tmp2 = NULL;
#line 146 "AllTray.vala"
	self->priv->_plist = g_list_append (self->priv->_plist, (_tmp2 = p, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)));
	a = (_vala_array_free (a, a_length1, (GDestroyNotify) g_free), NULL);
	(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
}


#line 192 "AllTray.vala"
static void _all_tray_program_sighandler_native_std_csignal_signal_handler (gint whichSignal) {
	all_tray_program_sighandler (whichSignal);
}


#line 149 "AllTray.vala"
static void all_tray_program_install_signal_handlers (AllTrayProgram* self) {
#line 149 "AllTray.vala"
	g_return_if_fail (self != NULL);
	/* For the moment, we _DO NOT_ use these.  We will soon.*/
#line 151 "AllTray.vala"
	return;
}


#line 160 "AllTray.vala"
gint all_tray_program_main (char** args, int args_length1) {
	AllTrayProgram* atray;
	AllTrayProgram* _tmp1;
	AllTrayProgram* _tmp0;
	gint _tmp2;
	atray = all_tray_program_new (&args, &args_length1);
	_tmp1 = NULL;
#line 162 "AllTray.vala"
	_tmp0 = NULL;
#line 162 "AllTray.vala"
	all_tray_program__instance = (_tmp1 = (_tmp0 = atray, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (all_tray_program__instance == NULL) ? NULL : (all_tray_program__instance = (g_object_unref (all_tray_program__instance), NULL)), _tmp1);
#line 164 "AllTray.vala"
	return (_tmp2 = all_tray_program_run (atray), (atray == NULL) ? NULL : (atray = (g_object_unref (atray), NULL)), _tmp2);
}


#line 160 "AllTray.vala"
int main (int argc, char ** argv) {
#line 160 "AllTray.vala"
	g_type_init ();
#line 160 "AllTray.vala"
	return all_tray_program_main (argv, argc);
}


#line 167 "AllTray.vala"
static void all_tray_program_cleanup_for_process (AllTrayProgram* self, AllTrayProcess* p) {
#line 167 "AllTray.vala"
	g_return_if_fail (self != NULL);
#line 167 "AllTray.vala"
	g_return_if_fail (p != NULL);
#line 168 "AllTray.vala"
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, "Cleaning up for child...");
#line 171 "AllTray.vala"
	self->priv->_plist = g_list_remove (self->priv->_plist, p);
#line 173 "AllTray.vala"
	if (g_list_length (self->priv->_plist) == 0) {
#line 174 "AllTray.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, "No more children. Dying.");
#line 177 "AllTray.vala"
		gtk_main_quit ();
	}
}


#line 181 "AllTray.vala"
static char** all_tray_program_get_command_line (AllTrayProgram* self, char** args, int args_length1, int* result_length1) {
	gint curItem;
	char** _tmp0;
	gint retval_size;
	gint retval_length1;
	char** retval;
	char** _tmp5;
#line 181 "AllTray.vala"
	g_return_val_if_fail (self != NULL, NULL);
	curItem = 0;
#line 183 "AllTray.vala"
	_tmp0 = NULL;
	retval = (_tmp0 = g_new0 (char*, args_length1 + 1), retval_length1 = args_length1, retval_size = retval_length1, _tmp0);
	{
		char** arg_collection;
		int arg_collection_length1;
		int arg_it;
#line 184 "AllTray.vala"
		arg_collection = args;
		arg_collection_length1 = args_length1;
		for (arg_it = 0; arg_it < args_length1; arg_it = arg_it + 1) {
			const char* _tmp4;
			char* arg;
#line 181 "AllTray.vala"
			_tmp4 = NULL;
			arg = (_tmp4 = arg_collection[arg_it], (_tmp4 == NULL) ? NULL : g_strdup (_tmp4));
			{
				char* _tmp3;
				char* *_tmp2;
				const char* _tmp1;
#line 185 "AllTray.vala"
				if (string_contains (arg, "alltray")) {
					arg = (g_free (arg), NULL);
#line 185 "AllTray.vala"
					continue;
				}
				_tmp3 = NULL;
#line 186 "AllTray.vala"
				_tmp1 = NULL;
#line 186 "AllTray.vala"
				_tmp2 = &retval[curItem++];
#line 186 "AllTray.vala"
				(*_tmp2) = (_tmp3 = (_tmp1 = arg, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), (*_tmp2) = (g_free ((*_tmp2)), NULL), _tmp3);
				arg = (g_free (arg), NULL);
			}
		}
	}
#line 189 "AllTray.vala"
	_tmp5 = NULL;
#line 189 "AllTray.vala"
	return (_tmp5 = retval, *result_length1 = retval_length1, _tmp5);
}


#line 192 "AllTray.vala"
static void all_tray_program_sighandler (gint caught_signal) {
	char* _tmp1;
	char* _tmp0;
	_tmp1 = NULL;
#line 99 "glib-2.0.vapi"
	_tmp0 = NULL;
#line 193 "AllTray.vala"
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Misc, ALL_TRAY_DEBUG_LEVEL_Information, _tmp1 = g_strconcat ("Caught signal: ", _tmp0 = g_strdup_printf ("%i", caught_signal), NULL));
	_tmp1 = (g_free (_tmp1), NULL);
	_tmp0 = (g_free (_tmp0), NULL);
}


char** all_tray_program_get_args (AllTrayProgram* self) {
	g_return_val_if_fail (self != NULL, NULL);
#line 33 "AllTray.vala"
	return self->priv->_args;
}


static void all_tray_program_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AllTrayProgram * self;
	gpointer boxed;
	self = ALL_TRAY_PROGRAM (object);
	switch (property_id) {
		case ALL_TRAY_PROGRAM_ARGS:
		g_value_set_pointer (value, all_tray_program_get_args (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void all_tray_program_class_init (AllTrayProgramClass * klass) {
	all_tray_program_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AllTrayProgramPrivate));
	G_OBJECT_CLASS (klass)->get_property = all_tray_program_get_property;
	G_OBJECT_CLASS (klass)->finalize = all_tray_program_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), ALL_TRAY_PROGRAM_ARGS, g_param_spec_pointer ("args", "args", "args", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void all_tray_program_instance_init (AllTrayProgram * self) {
	self->priv = ALL_TRAY_PROGRAM_GET_PRIVATE (self);
}


static void all_tray_program_finalize (GObject* obj) {
	AllTrayProgram * self;
	self = ALL_TRAY_PROGRAM (obj);
	self->priv->_args = (_vala_array_free (self->priv->_args, self->priv->_args_length1, (GDestroyNotify) g_free), NULL);
	self->priv->_cleanArgs = (_vala_array_free (self->priv->_cleanArgs, self->priv->_cleanArgs_length1, (GDestroyNotify) g_free), NULL);
	(self->priv->_plist == NULL) ? NULL : (self->priv->_plist = (_g_list_free_g_object_unref (self->priv->_plist), NULL));
	G_OBJECT_CLASS (all_tray_program_parent_class)->finalize (obj);
}


GType all_tray_program_get_type (void) {
	static GType all_tray_program_type_id = 0;
	if (all_tray_program_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayProgramClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_program_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayProgram), 0, (GInstanceInitFunc) all_tray_program_instance_init, NULL };
		all_tray_program_type_id = g_type_register_static (G_TYPE_OBJECT, "AllTrayProgram", &g_define_type_info, 0);
	}
	return all_tray_program_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




