/*
 * Application.vala - Application-based window management for AllTray
 * Copyright (c) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3.0 as published by the Free Software Foundation
 */

#include "Application.h"
#include <gdk/gdkx.h>
#include <gdk/gdk.h>
#include <x11_glue.h>
#include <stdlib.h>
#include <string.h>
#include "Debug.h"




static char* bool_to_string (gboolean self);
struct _AllTrayApplicationPrivate {
	WnckApplication* _wnckApp;
	AllTrayProcess* _process;
	gboolean _appVisible;
	gulong _xWin;
};

#define ALL_TRAY_APPLICATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ALL_TRAY_TYPE_APPLICATION, AllTrayApplicationPrivate))
enum  {
	ALL_TRAY_APPLICATION_DUMMY_PROPERTY,
	ALL_TRAY_APPLICATION_WNCK_APP
};
static gboolean _all_tray_application_get_wapp_gsource_func (gpointer self);
static gulong all_tray_application_get_xwin_from_pid (AllTrayApplication* self, GPid pid);
static gpointer all_tray_application_parent_class = NULL;
static void all_tray_application_finalize (GObject* obj);



#line 35 "glib-2.0.vapi"
static char* bool_to_string (gboolean self) {
#line 36 "glib-2.0.vapi"
	if (self) {
#line 37 "glib-2.0.vapi"
		return g_strdup ("true");
	} else {
#line 39 "glib-2.0.vapi"
		return g_strdup ("false");
	}
}


GQuark all_tray_application_error_quark (void) {
	return g_quark_from_static_string ("all_tray_application_error-quark");
}


#line 40 "Application.vala"
static gboolean _all_tray_application_get_wapp_gsource_func (gpointer self) {
	return all_tray_application_get_wapp (self);
}


#line 25 "Application.vala"
AllTrayApplication* all_tray_application_construct (GType object_type, AllTrayProcess* p) {
	AllTrayApplication * self;
	AllTrayProcess* _tmp1;
	AllTrayProcess* _tmp0;
	WnckApplication* _tmp3;
	WnckApplication* _tmp2;
#line 25 "Application.vala"
	g_return_val_if_fail (p != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
#line 26 "Application.vala"
	_tmp0 = NULL;
#line 26 "Application.vala"
	self->priv->_process = (_tmp1 = (_tmp0 = p, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_process == NULL) ? NULL : (self->priv->_process = (g_object_unref (self->priv->_process), NULL)), _tmp1);
#line 27 "Application.vala"
	self->priv->_xWin = all_tray_application_get_xwin_from_pid (self, all_tray_process_get_pid (p));
	_tmp3 = NULL;
#line 29 "Application.vala"
	_tmp2 = NULL;
#line 29 "Application.vala"
	self->priv->_wnckApp = (_tmp3 = (_tmp2 = wnck_application_get (self->priv->_xWin), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_wnckApp == NULL) ? NULL : (self->priv->_wnckApp = (g_object_unref (self->priv->_wnckApp), NULL)), _tmp3);
#line 30 "Application.vala"
	if (self->priv->_wnckApp == NULL) {
#line 31 "Application.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, "No App yet. Waiting for one.");
#line 34 "Application.vala"
		g_timeout_add ((guint) 50, _all_tray_application_get_wapp_gsource_func, self);
	}
#line 37 "Application.vala"
	self->priv->_appVisible = TRUE;
	return self;
}


#line 25 "Application.vala"
AllTrayApplication* all_tray_application_new (AllTrayProcess* p) {
#line 25 "Application.vala"
	return all_tray_application_construct (ALL_TRAY_TYPE_APPLICATION, p);
}


#line 40 "Application.vala"
gboolean all_tray_application_get_wapp (AllTrayApplication* self) {
	WnckApplication* _tmp1;
	WnckApplication* _tmp0;
#line 40 "Application.vala"
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp1 = NULL;
#line 41 "Application.vala"
	_tmp0 = NULL;
#line 41 "Application.vala"
	self->priv->_wnckApp = (_tmp1 = (_tmp0 = wnck_application_get (self->priv->_xWin), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_wnckApp == NULL) ? NULL : (self->priv->_wnckApp = (g_object_unref (self->priv->_wnckApp), NULL)), _tmp1);
#line 42 "Application.vala"
	if (self->priv->_wnckApp == NULL) {
#line 43 "Application.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, "No App yet. Waiting for one.");
#line 46 "Application.vala"
		return TRUE;
	} else {
#line 48 "Application.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, "Got the app.");
#line 51 "Application.vala"
		return FALSE;
	}
}


#line 55 "Application.vala"
void all_tray_application_toggle_visibility (AllTrayApplication* self) {
	GList* windows;
	GString* msg;
#line 55 "Application.vala"
	g_return_if_fail (self != NULL);
#line 56 "Application.vala"
	self->priv->_appVisible = !self->priv->_appVisible;
	windows = wnck_application_get_windows (self->priv->_wnckApp);
	msg = g_string_new ("");
	{
		GList* w_collection;
		GList* w_it;
#line 61 "Application.vala"
		w_collection = windows;
		for (w_it = w_collection; w_it != NULL; w_it = w_it->next) {
			WnckWindow* _tmp1;
			WnckWindow* w;
#line 58 "Application.vala"
			_tmp1 = NULL;
#line 61 "Application.vala"
			w = (_tmp1 = (WnckWindow*) w_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
			{
				char* _tmp0;
#line 62 "Application.vala"
				g_string_truncate (msg, (gsize) 0);
#line 35 "glib-2.0.vapi"
				_tmp0 = NULL;
#line 63 "Application.vala"
				g_string_append_printf (msg, "Setting window 0x%08lx visibility to %s", wnck_window_get_xid (w), _tmp0 = bool_to_string (self->priv->_appVisible));
				_tmp0 = (g_free (_tmp0), NULL);
#line 66 "Application.vala"
				if (self->priv->_appVisible) {
#line 67 "Application.vala"
					wnck_window_unminimize (w, (guint32) GDK_CURRENT_TIME);
#line 68 "Application.vala"
					wnck_window_set_skip_tasklist (w, FALSE);
				} else {
#line 70 "Application.vala"
					wnck_window_minimize (w);
#line 71 "Application.vala"
					wnck_window_set_skip_tasklist (w, TRUE);
				}
#line 73 "Application.vala"
				all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, msg->str);
				(w == NULL) ? NULL : (w = (g_object_unref (w), NULL));
			}
		}
	}
	(msg == NULL) ? NULL : (msg = (g_string_free (msg, TRUE), NULL));
}


#line 79 "Application.vala"
static gulong all_tray_application_get_xwin_from_pid (AllTrayApplication* self, GPid pid) {
	void* Display;
	gulong xid;
#line 79 "Application.vala"
	g_return_val_if_fail (self != NULL, 0UL);
	Display = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
	xid = alltray_get_x11_window_from_pid (pid, Display);
#line 85 "Application.vala"
	while (xid == 0) {
#line 86 "Application.vala"
		xid = alltray_get_x11_window_from_pid (pid, Display);
	}
#line 89 "Application.vala"
	return xid;
}


WnckApplication* all_tray_application_get_wnck_app (AllTrayApplication* self) {
	g_return_val_if_fail (self != NULL, NULL);
#line 21 "Application.vala"
	return self->priv->_wnckApp;
}


static void all_tray_application_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AllTrayApplication * self;
	gpointer boxed;
	self = ALL_TRAY_APPLICATION (object);
	switch (property_id) {
		case ALL_TRAY_APPLICATION_WNCK_APP:
		g_value_set_object (value, all_tray_application_get_wnck_app (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void all_tray_application_class_init (AllTrayApplicationClass * klass) {
	all_tray_application_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AllTrayApplicationPrivate));
	G_OBJECT_CLASS (klass)->get_property = all_tray_application_get_property;
	G_OBJECT_CLASS (klass)->finalize = all_tray_application_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), ALL_TRAY_APPLICATION_WNCK_APP, g_param_spec_object ("wnck-app", "wnck-app", "wnck-app", WNCK_TYPE_APPLICATION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void all_tray_application_instance_init (AllTrayApplication * self) {
	self->priv = ALL_TRAY_APPLICATION_GET_PRIVATE (self);
}


static void all_tray_application_finalize (GObject* obj) {
	AllTrayApplication * self;
	self = ALL_TRAY_APPLICATION (obj);
	(self->priv->_wnckApp == NULL) ? NULL : (self->priv->_wnckApp = (g_object_unref (self->priv->_wnckApp), NULL));
	(self->priv->_process == NULL) ? NULL : (self->priv->_process = (g_object_unref (self->priv->_process), NULL));
	G_OBJECT_CLASS (all_tray_application_parent_class)->finalize (obj);
}


GType all_tray_application_get_type (void) {
	static GType all_tray_application_type_id = 0;
	if (all_tray_application_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayApplicationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_application_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayApplication), 0, (GInstanceInitFunc) all_tray_application_instance_init, NULL };
		all_tray_application_type_id = g_type_register_static (G_TYPE_OBJECT, "AllTrayApplication", &g_define_type_info, 0);
	}
	return all_tray_application_type_id;
}




