/*
 * Debug.vala - Debugging system for AllTray.
 * Copyright (C) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3 as published by the Free Software Foundation.
 */

#include "Debug.h"
#include <time.h>
#include <stdio.h>
#include <gobject/gvaluecollector.h>




static char* g_time_format (struct tm *self, const char* format);
enum  {
	ALL_TRAY_DEBUG_NOTIFICATION_DUMMY_PROPERTY
};
static AllTrayDebugSubsystem all_tray_debug_notification__subsys = ALL_TRAY_DEBUG_SUBSYSTEM_None;
static gboolean all_tray_debug_notification__enabled = FALSE;
static char* all_tray_debug_notification_subsys_to_string (AllTrayDebugSubsystem subsys);
static char* all_tray_debug_notification_lvl_to_string (AllTrayDebugLevel lvl);
static gpointer all_tray_debug_notification_parent_class = NULL;
static void all_tray_debug_notification_finalize (AllTrayDebugNotification* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 1890 "glib-2.0.vapi"
static char* g_time_format (struct tm *self, const char* format) {
	gchar* _tmp0;
	gint buffer_size;
	gint buffer_length1;
	gchar* buffer;
	const char* _tmp1;
	char* _tmp2;
#line 1890 "glib-2.0.vapi"
	g_return_val_if_fail (format != NULL, NULL);
#line 1891 "glib-2.0.vapi"
	_tmp0 = NULL;
	buffer = (_tmp0 = g_new0 (gchar, 64), buffer_length1 = 64, buffer_size = buffer_length1, _tmp0);
#line 1892 "glib-2.0.vapi"
	strftime (buffer, buffer_length1, format, &(*self));
#line 1893 "glib-2.0.vapi"
	_tmp1 = NULL;
#line 1893 "glib-2.0.vapi"
	_tmp2 = NULL;
#line 1893 "glib-2.0.vapi"
	return (_tmp2 = (_tmp1 = (const char*) buffer, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), buffer = (g_free (buffer), NULL), _tmp2);
}



GType all_tray_debug_subsystem_get_type (void) {
	static GType all_tray_debug_subsystem_type_id = 0;
	if (G_UNLIKELY (all_tray_debug_subsystem_type_id == 0)) {
		static const GEnumValue values[] = {{ALL_TRAY_DEBUG_SUBSYSTEM_None, "ALL_TRAY_DEBUG_SUBSYSTEM_None", "none"}, {ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine, "ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine", "commandline"}, {ALL_TRAY_DEBUG_SUBSYSTEM_Display, "ALL_TRAY_DEBUG_SUBSYSTEM_Display", "display"}, {ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon, "ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon", "trayicon"}, {ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager, "ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager", "windowmanager"}, {ALL_TRAY_DEBUG_SUBSYSTEM_Process, "ALL_TRAY_DEBUG_SUBSYSTEM_Process", "process"}, {ALL_TRAY_DEBUG_SUBSYSTEM_Misc, "ALL_TRAY_DEBUG_SUBSYSTEM_Misc", "misc"}, {ALL_TRAY_DEBUG_SUBSYSTEM_Bug, "ALL_TRAY_DEBUG_SUBSYSTEM_Bug", "bug"}, {ALL_TRAY_DEBUG_SUBSYSTEM_Application, "ALL_TRAY_DEBUG_SUBSYSTEM_Application", "application"}, {0, NULL, NULL}};
		all_tray_debug_subsystem_type_id = g_enum_register_static ("AllTrayDebugSubsystem", values);
	}
	return all_tray_debug_subsystem_type_id;
}



GType all_tray_debug_level_get_type (void) {
	static GType all_tray_debug_level_type_id = 0;
	if (G_UNLIKELY (all_tray_debug_level_type_id == 0)) {
		static const GEnumValue values[] = {{ALL_TRAY_DEBUG_LEVEL_Information, "ALL_TRAY_DEBUG_LEVEL_Information", "information"}, {ALL_TRAY_DEBUG_LEVEL_Warning, "ALL_TRAY_DEBUG_LEVEL_Warning", "warning"}, {ALL_TRAY_DEBUG_LEVEL_Error, "ALL_TRAY_DEBUG_LEVEL_Error", "error"}, {ALL_TRAY_DEBUG_LEVEL_Fatal, "ALL_TRAY_DEBUG_LEVEL_Fatal", "fatal"}, {0, NULL, NULL}};
		all_tray_debug_level_type_id = g_enum_register_static ("AllTrayDebugLevel", values);
	}
	return all_tray_debug_level_type_id;
}


#line 32 "Debug.vala"
void all_tray_debug_notification_init (void) {
	const char* _tmp0;
	char* enabled_subsystems;
#line 33 "Debug.vala"
	all_tray_debug_notification__enabled = TRUE;
#line 35 "Debug.vala"
	_tmp0 = NULL;
	enabled_subsystems = (_tmp0 = g_getenv ("ALLTRAY_DEBUG"), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
#line 37 "Debug.vala"
	if (enabled_subsystems != NULL) {
		char** _tmp2;
		gint values_size;
		gint values_length1;
		char** _tmp1;
		char** values;
#line 38 "Debug.vala"
		_tmp2 = NULL;
		_tmp1 = NULL;
		values = (_tmp2 = _tmp1 = g_strsplit (enabled_subsystems, " ", 0), values_length1 = _vala_array_length (_tmp1), values_size = values_length1, _tmp2);
		{
			char** tmpSubsys_collection;
			int tmpSubsys_collection_length1;
			int tmpSubsys_it;
#line 39 "Debug.vala"
			tmpSubsys_collection = values;
			tmpSubsys_collection_length1 = values_length1;
			for (tmpSubsys_it = 0; tmpSubsys_it < values_length1; tmpSubsys_it = tmpSubsys_it + 1) {
				const char* _tmp5;
				char* tmpSubsys;
#line 38 "Debug.vala"
				_tmp5 = NULL;
				tmpSubsys = (_tmp5 = tmpSubsys_collection[tmpSubsys_it], (_tmp5 == NULL) ? NULL : g_strdup (_tmp5));
				{
					GQuark _tmp4;
					char* _tmp3;
#line 40 "Debug.vala"
					_tmp3 = NULL;
					static GQuark _tmp4_label0 = 0;
					static GQuark _tmp4_label1 = 0;
					static GQuark _tmp4_label2 = 0;
					static GQuark _tmp4_label3 = 0;
					static GQuark _tmp4_label4 = 0;
					static GQuark _tmp4_label5 = 0;
					static GQuark _tmp4_label6 = 0;
					static GQuark _tmp4_label7 = 0;
					static GQuark _tmp4_label8 = 0;
					_tmp3 = tmpSubsys;
					_tmp4 = (NULL == _tmp3) ? 0 : g_quark_from_string (_tmp3);
					if (_tmp4 == ((0 != _tmp4_label0) ? _tmp4_label0 : (_tmp4_label0 = g_quark_from_static_string ("CL"))))
					do {
#line 42 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine;
#line 43 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label1) ? _tmp4_label1 : (_tmp4_label1 = g_quark_from_static_string ("DISPLAY"))))
					do {
#line 45 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_Display;
#line 46 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label2) ? _tmp4_label2 : (_tmp4_label2 = g_quark_from_static_string ("SYSTRAY"))))
					do {
#line 48 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon;
#line 49 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label3) ? _tmp4_label3 : (_tmp4_label3 = g_quark_from_static_string ("WM"))))
					do {
#line 51 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager;
#line 52 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label4) ? _tmp4_label4 : (_tmp4_label4 = g_quark_from_static_string ("PROCESS"))))
					do {
#line 54 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_Process;
#line 55 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label5) ? _tmp4_label5 : (_tmp4_label5 = g_quark_from_static_string ("MISC"))))
					do {
#line 57 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_Misc;
#line 58 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label6) ? _tmp4_label6 : (_tmp4_label6 = g_quark_from_static_string ("BUG"))))
					do {
#line 60 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_Bug;
#line 61 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label7) ? _tmp4_label7 : (_tmp4_label7 = g_quark_from_static_string ("APP"))))
					do {
#line 63 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | ALL_TRAY_DEBUG_SUBSYSTEM_Application;
#line 64 "Debug.vala"
						break;
					} while (0); else if (_tmp4 == ((0 != _tmp4_label8) ? _tmp4_label8 : (_tmp4_label8 = g_quark_from_static_string ("ALL"))))
					do {
#line 66 "Debug.vala"
						all_tray_debug_notification__subsys = all_tray_debug_notification__subsys | (((((((ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine | ALL_TRAY_DEBUG_SUBSYSTEM_Display) | ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon) | ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager) | ALL_TRAY_DEBUG_SUBSYSTEM_Process) | ALL_TRAY_DEBUG_SUBSYSTEM_Misc) | ALL_TRAY_DEBUG_SUBSYSTEM_Bug) | ALL_TRAY_DEBUG_SUBSYSTEM_Application);
#line 74 "Debug.vala"
						break;
					} while (0); else
					do {
#line 76 "Debug.vala"
						g_warning ("Debug.vala:76: Unrecognized value '%s' in ALLTRAY_DEBUG", tmpSubsys);
#line 78 "Debug.vala"
						break;
					} while (0);
					tmpSubsys = (g_free (tmpSubsys), NULL);
				}
			}
		}
#line 82 "Debug.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Misc, ALL_TRAY_DEBUG_LEVEL_Information, "Debugging subsystem setup complete.");
		values = (_vala_array_free (values, values_length1, (GDestroyNotify) g_free), NULL);
	} else {
		gboolean _tmp6;
		_tmp6 = FALSE;
#line 84 "Debug.vala"
		if (enabled_subsystems == NULL) {
#line 84 "Debug.vala"
			_tmp6 = TRUE;
		} else {
#line 85 "Debug.vala"
			_tmp6 = _vala_strcmp0 (enabled_subsystems, "") == 0;
		}
#line 84 "Debug.vala"
		if (_tmp6) {
#line 86 "Debug.vala"
			g_warning ("Debug.vala:86: ALLTRAY_DEBUG not set!");
		}
	}
	enabled_subsystems = (g_free (enabled_subsystems), NULL);
}


#line 90 "Debug.vala"
void all_tray_debug_notification_emit (AllTrayDebugSubsystem subsys, AllTrayDebugLevel lvl, const char* msg) {
	char* cur_time;
	char* _tmp4;
	struct tm _tmp3 = {0};
	time_t _tmp2 = {0};
	time_t _tmp0 = {0};
	struct tm _tmp1 = {0};
	char* lvl_str;
	char* subsys_str;
#line 90 "Debug.vala"
	g_return_if_fail (msg != NULL);
	/* Bail out early when we should not run.*/
#line 92 "Debug.vala"
	if (!all_tray_debug_notification__enabled) {
#line 92 "Debug.vala"
		return;
	}
#line 93 "Debug.vala"
	if ((all_tray_debug_notification__subsys & subsys) != subsys) {
#line 93 "Debug.vala"
		return;
	}
	cur_time = NULL;
	_tmp4 = NULL;
#line 97 "Debug.vala"
	cur_time = (_tmp4 = g_time_format ((_tmp3 = (localtime_r ((_tmp2 = (time (&_tmp0), _tmp0), &_tmp2), &_tmp1), _tmp1), &_tmp3), "%F %T %Z"), cur_time = (g_free (cur_time), NULL), _tmp4);
	lvl_str = all_tray_debug_notification_lvl_to_string (lvl);
	subsys_str = all_tray_debug_notification_subsys_to_string (subsys);
#line 101 "Debug.vala"
	fprintf (stderr, "[%s] %s/%s: %s\n", cur_time, lvl_str, subsys_str, msg);
#line 104 "Debug.vala"
	if (lvl == ALL_TRAY_DEBUG_LEVEL_Fatal) {
#line 105 "Debug.vala"
		all_tray_debug_notification_emit (subsys, ALL_TRAY_DEBUG_LEVEL_Information, "Exiting: encountered fatal error.");
#line 107 "Debug.vala"
		abort ();
	}
	cur_time = (g_free (cur_time), NULL);
	lvl_str = (g_free (lvl_str), NULL);
	subsys_str = (g_free (subsys_str), NULL);
}


#line 111 "Debug.vala"
static char* all_tray_debug_notification_subsys_to_string (AllTrayDebugSubsystem subsys) {
	char* retval;
	retval = g_strdup ("");
#line 114 "Debug.vala"
	switch (subsys) {
		case ALL_TRAY_DEBUG_SUBSYSTEM_None:
		{
			char* _tmp0;
			_tmp0 = NULL;
#line 116 "Debug.vala"
			retval = (_tmp0 = g_strdup ("None"), retval = (g_free (retval), NULL), _tmp0);
#line 117 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine:
		{
			char* _tmp1;
			_tmp1 = NULL;
#line 120 "Debug.vala"
			retval = (_tmp1 = g_strdup ("CL"), retval = (g_free (retval), NULL), _tmp1);
#line 121 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_Display:
		{
			char* _tmp2;
			_tmp2 = NULL;
#line 124 "Debug.vala"
			retval = (_tmp2 = g_strdup ("DISPLAY"), retval = (g_free (retval), NULL), _tmp2);
#line 125 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon:
		{
			char* _tmp3;
			_tmp3 = NULL;
#line 128 "Debug.vala"
			retval = (_tmp3 = g_strdup ("TRAY"), retval = (g_free (retval), NULL), _tmp3);
#line 129 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager:
		{
			char* _tmp4;
			_tmp4 = NULL;
#line 132 "Debug.vala"
			retval = (_tmp4 = g_strdup ("WM"), retval = (g_free (retval), NULL), _tmp4);
#line 133 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_Process:
		{
			char* _tmp5;
			_tmp5 = NULL;
#line 136 "Debug.vala"
			retval = (_tmp5 = g_strdup ("PROCESS"), retval = (g_free (retval), NULL), _tmp5);
#line 137 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_Misc:
		{
			char* _tmp6;
			_tmp6 = NULL;
#line 140 "Debug.vala"
			retval = (_tmp6 = g_strdup ("MISC"), retval = (g_free (retval), NULL), _tmp6);
#line 141 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_Application:
		{
			char* _tmp7;
			_tmp7 = NULL;
#line 144 "Debug.vala"
			retval = (_tmp7 = g_strdup ("APP"), retval = (g_free (retval), NULL), _tmp7);
#line 145 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_SUBSYSTEM_Bug:
		{
			char* _tmp8;
			_tmp8 = NULL;
#line 148 "Debug.vala"
			retval = (_tmp8 = g_strdup ("BUG"), retval = (g_free (retval), NULL), _tmp8);
#line 149 "Debug.vala"
			break;
		}
		default:
		{
#line 152 "Debug.vala"
			g_error ("Debug.vala:152: BUG: Reached default, should not be possible.");
#line 153 "Debug.vala"
			break;
		}
	}
#line 156 "Debug.vala"
	return retval;
}


#line 159 "Debug.vala"
static char* all_tray_debug_notification_lvl_to_string (AllTrayDebugLevel lvl) {
	char* retval;
	retval = g_strdup ("");
#line 162 "Debug.vala"
	switch (lvl) {
		case ALL_TRAY_DEBUG_LEVEL_Information:
		{
			char* _tmp0;
			_tmp0 = NULL;
#line 164 "Debug.vala"
			retval = (_tmp0 = g_strdup ("INFO"), retval = (g_free (retval), NULL), _tmp0);
#line 165 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_LEVEL_Warning:
		{
			char* _tmp1;
			_tmp1 = NULL;
#line 168 "Debug.vala"
			retval = (_tmp1 = g_strdup ("WARN"), retval = (g_free (retval), NULL), _tmp1);
#line 169 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_LEVEL_Error:
		{
			char* _tmp2;
			_tmp2 = NULL;
#line 172 "Debug.vala"
			retval = (_tmp2 = g_strdup ("ERR"), retval = (g_free (retval), NULL), _tmp2);
#line 173 "Debug.vala"
			break;
		}
		case ALL_TRAY_DEBUG_LEVEL_Fatal:
		{
			char* _tmp3;
			_tmp3 = NULL;
#line 176 "Debug.vala"
			retval = (_tmp3 = g_strdup ("FATAL"), retval = (g_free (retval), NULL), _tmp3);
#line 177 "Debug.vala"
			break;
		}
		default:
		{
#line 180 "Debug.vala"
			g_error ("Debug.vala:180: BUG: Reached default, should not be possible.");
#line 181 "Debug.vala"
			break;
		}
	}
#line 184 "Debug.vala"
	return retval;
}


#line 28 "Debug.vala"
AllTrayDebugNotification* all_tray_debug_notification_construct (GType object_type) {
	AllTrayDebugNotification* self;
	self = (AllTrayDebugNotification*) g_type_create_instance (object_type);
	return self;
}


#line 28 "Debug.vala"
AllTrayDebugNotification* all_tray_debug_notification_new (void) {
#line 28 "Debug.vala"
	return all_tray_debug_notification_construct (ALL_TRAY_DEBUG_TYPE_NOTIFICATION);
}


static void all_tray_debug_value_notification_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void all_tray_debug_value_notification_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		all_tray_debug_notification_unref (value->data[0].v_pointer);
	}
}


static void all_tray_debug_value_notification_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = all_tray_debug_notification_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer all_tray_debug_value_notification_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* all_tray_debug_value_notification_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		AllTrayDebugNotification* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = all_tray_debug_notification_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* all_tray_debug_value_notification_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	AllTrayDebugNotification** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = all_tray_debug_notification_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* all_tray_debug_param_spec_notification (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	AllTrayDebugParamSpecNotification* spec;
	g_return_val_if_fail (g_type_is_a (object_type, ALL_TRAY_DEBUG_TYPE_NOTIFICATION), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer all_tray_debug_value_get_notification (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, ALL_TRAY_DEBUG_TYPE_NOTIFICATION), NULL);
	return value->data[0].v_pointer;
}


void all_tray_debug_value_set_notification (GValue* value, gpointer v_object) {
	AllTrayDebugNotification* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, ALL_TRAY_DEBUG_TYPE_NOTIFICATION));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, ALL_TRAY_DEBUG_TYPE_NOTIFICATION));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		all_tray_debug_notification_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		all_tray_debug_notification_unref (old);
	}
}


static void all_tray_debug_notification_class_init (AllTrayDebugNotificationClass * klass) {
	all_tray_debug_notification_parent_class = g_type_class_peek_parent (klass);
	ALL_TRAY_DEBUG_NOTIFICATION_CLASS (klass)->finalize = all_tray_debug_notification_finalize;
}


static void all_tray_debug_notification_instance_init (AllTrayDebugNotification * self) {
	self->ref_count = 1;
}


static void all_tray_debug_notification_finalize (AllTrayDebugNotification* obj) {
	AllTrayDebugNotification * self;
	self = ALL_TRAY_DEBUG_NOTIFICATION (obj);
}


GType all_tray_debug_notification_get_type (void) {
	static GType all_tray_debug_notification_type_id = 0;
	if (all_tray_debug_notification_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { all_tray_debug_value_notification_init, all_tray_debug_value_notification_free_value, all_tray_debug_value_notification_copy_value, all_tray_debug_value_notification_peek_pointer, "p", all_tray_debug_value_notification_collect_value, "p", all_tray_debug_value_notification_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayDebugNotificationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_debug_notification_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayDebugNotification), 0, (GInstanceInitFunc) all_tray_debug_notification_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		all_tray_debug_notification_type_id = g_type_register_fundamental (g_type_fundamental_next (), "AllTrayDebugNotification", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return all_tray_debug_notification_type_id;
}


gpointer all_tray_debug_notification_ref (gpointer instance) {
	AllTrayDebugNotification* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void all_tray_debug_notification_unref (gpointer instance) {
	AllTrayDebugNotification* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		ALL_TRAY_DEBUG_NOTIFICATION_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




