/*
 * Debug.vala - Debugging system for AllTray.
 * Copyright (C) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3 as published by the Free Software Foundation.
 */

#ifndef __DEBUG_H__
#define __DEBUG_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define ALL_TRAY_DEBUG_TYPE_SUBSYSTEM (all_tray_debug_subsystem_get_type ())

#define ALL_TRAY_DEBUG_TYPE_LEVEL (all_tray_debug_level_get_type ())

#define ALL_TRAY_DEBUG_TYPE_NOTIFICATION (all_tray_debug_notification_get_type ())
#define ALL_TRAY_DEBUG_NOTIFICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ALL_TRAY_DEBUG_TYPE_NOTIFICATION, AllTrayDebugNotification))
#define ALL_TRAY_DEBUG_NOTIFICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ALL_TRAY_DEBUG_TYPE_NOTIFICATION, AllTrayDebugNotificationClass))
#define ALL_TRAY_DEBUG_IS_NOTIFICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ALL_TRAY_DEBUG_TYPE_NOTIFICATION))
#define ALL_TRAY_DEBUG_IS_NOTIFICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ALL_TRAY_DEBUG_TYPE_NOTIFICATION))
#define ALL_TRAY_DEBUG_NOTIFICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ALL_TRAY_DEBUG_TYPE_NOTIFICATION, AllTrayDebugNotificationClass))

typedef struct _AllTrayDebugNotification AllTrayDebugNotification;
typedef struct _AllTrayDebugNotificationClass AllTrayDebugNotificationClass;
typedef struct _AllTrayDebugNotificationPrivate AllTrayDebugNotificationPrivate;
typedef struct _AllTrayDebugParamSpecNotification AllTrayDebugParamSpecNotification;

typedef enum  {
	ALL_TRAY_DEBUG_SUBSYSTEM_None = 0x00,
	ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine = 0x01,
	ALL_TRAY_DEBUG_SUBSYSTEM_Display = 0x02,
	ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon = 0x04,
	ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager = 0x08,
	ALL_TRAY_DEBUG_SUBSYSTEM_Process = 0x10,
	ALL_TRAY_DEBUG_SUBSYSTEM_Misc = 0x20,
	ALL_TRAY_DEBUG_SUBSYSTEM_Bug = 0x40,
	ALL_TRAY_DEBUG_SUBSYSTEM_Application = 0x80
} AllTrayDebugSubsystem;

typedef enum  {
	ALL_TRAY_DEBUG_LEVEL_Information,
	ALL_TRAY_DEBUG_LEVEL_Warning,
	ALL_TRAY_DEBUG_LEVEL_Error,
	ALL_TRAY_DEBUG_LEVEL_Fatal
} AllTrayDebugLevel;

struct _AllTrayDebugNotification {
	GTypeInstance parent_instance;
	volatile int ref_count;
	AllTrayDebugNotificationPrivate * priv;
};

struct _AllTrayDebugNotificationClass {
	GTypeClass parent_class;
	void (*finalize) (AllTrayDebugNotification *self);
};

struct _AllTrayDebugParamSpecNotification {
	GParamSpec parent_instance;
};


GType all_tray_debug_subsystem_get_type (void);
GType all_tray_debug_level_get_type (void);
void all_tray_debug_notification_init (void);
void all_tray_debug_notification_emit (AllTrayDebugSubsystem subsys, AllTrayDebugLevel lvl, const char* msg);
AllTrayDebugNotification* all_tray_debug_notification_construct (GType object_type);
AllTrayDebugNotification* all_tray_debug_notification_new (void);
GParamSpec* all_tray_debug_param_spec_notification (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer all_tray_debug_value_get_notification (const GValue* value);
void all_tray_debug_value_set_notification (GValue* value, gpointer v_object);
GType all_tray_debug_notification_get_type (void);
gpointer all_tray_debug_notification_ref (gpointer instance);
void all_tray_debug_notification_unref (gpointer instance);


G_END_DECLS

#endif
