/*
 * Process.vala - Child process management
 * Copyright (c) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3.0 as published by the Free Software Foundation
 */

#include "Process.h"
#include <gtk/gtk.h>
#include <stdio.h>
#include "Application.h"
#include "TrayIcon.h"
#include "Debug.h"




struct _AllTrayProcessPrivate {
	GtkStatusIcon* _statusIcon;
	GPid _child;
	gboolean _visible;
	gboolean _running;
	char** _argv;
	gint _argv_length1;
	gint _argv_size;
	AllTrayApplication* _app;
	AllTrayTrayIcon* _trayIcon;
};

#define ALL_TRAY_PROCESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ALL_TRAY_TYPE_PROCESS, AllTrayProcessPrivate))
enum  {
	ALL_TRAY_PROCESS_DUMMY_PROPERTY,
	ALL_TRAY_PROCESS_RUNNING
};
static char** _vala_array_dup3 (char** self, int length);
static void _all_tray_process_child_died_gchild_watch_func (GPid pid, gint status, gpointer self);
static gpointer all_tray_process_parent_class = NULL;
static void all_tray_process_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



GQuark all_tray_process_error_quark (void) {
	return g_quark_from_static_string ("all_tray_process_error-quark");
}


static char** _vala_array_dup3 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp0;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp0 = self[i], (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	}
	return result;
}


/* XXX: Not for now, bug in Vala 0.[67].0

public Pid pid {
get {
return(_child);
}
}
*/
#line 40 "Process.vala"
AllTrayProcess* all_tray_process_construct (GType object_type, char** argv, int argv_length1) {
	AllTrayProcess * self;
	char** _tmp2;
	char** _tmp1;
	const char* _tmp0;
	self = g_object_newv (object_type, 0, NULL);
	_tmp2 = NULL;
#line 41 "Process.vala"
	_tmp1 = NULL;
#line 19 "Process.vala"
	_tmp0 = NULL;
#line 41 "Process.vala"
	self->priv->_argv = (_tmp2 = (_tmp1 = argv, (_tmp1 == NULL) ? ((gpointer) _tmp1) : _vala_array_dup3 (_tmp1, argv_length1)), self->priv->_argv = (_vala_array_free (self->priv->_argv, self->priv->_argv_length1, (GDestroyNotify) g_free), NULL), self->priv->_argv_length1 = argv_length1, self->priv->_argv_size = self->priv->_argv_length1, _tmp2);
#line 42 "Process.vala"
	self->priv->_visible = TRUE;
	return self;
}


#line 40 "Process.vala"
AllTrayProcess* all_tray_process_new (char** argv, int argv_length1) {
#line 40 "Process.vala"
	return all_tray_process_construct (ALL_TRAY_TYPE_PROCESS, argv, argv_length1);
}


#line 45 "Process.vala"
GPid all_tray_process_get_pid (AllTrayProcess* self) {
#line 45 "Process.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 46 "Process.vala"
	return self->priv->_child;
}


#line 72 "Process.vala"
static void _all_tray_process_child_died_gchild_watch_func (GPid pid, gint status, gpointer self) {
	all_tray_process_child_died (self);
}


#line 49 "Process.vala"
void all_tray_process_run (AllTrayProcess* self) {
	GError * inner_error;
	AllTrayApplication* _tmp0;
	AllTrayTrayIcon* _tmp1;
#line 49 "Process.vala"
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		GString* msg;
#line 51 "Process.vala"
		g_spawn_async (NULL, self->priv->_argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, NULL, NULL, &self->priv->_child, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch3_g_spawn_error;
			}
			goto __finally3;
		}
#line 55 "Process.vala"
		self->priv->_running = TRUE;
#line 56 "Process.vala"
		g_child_watch_add (self->priv->_child, _all_tray_process_child_died_gchild_watch_func, self);
		msg = g_string_new ("");
#line 58 "Process.vala"
		g_string_append_printf (msg, "Child process %d (%s) now running.", (gint) self->priv->_child, self->priv->_argv[0]);
#line 60 "Process.vala"
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, msg->str);
		(msg == NULL) ? NULL : (msg = (g_string_free (msg, TRUE), NULL));
	}
	goto __finally3;
	__catch3_g_spawn_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
#line 64 "Process.vala"
			fprintf (stdout, "An error occurred: %s\n", e->message);
#line 65 "Process.vala"
			self->priv->_running = FALSE;
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally3:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	_tmp0 = NULL;
#line 68 "Process.vala"
	self->priv->_app = (_tmp0 = all_tray_application_new (self), (self->priv->_app == NULL) ? NULL : (self->priv->_app = (g_object_unref (self->priv->_app), NULL)), _tmp0);
	_tmp1 = NULL;
#line 69 "Process.vala"
	self->priv->_trayIcon = (_tmp1 = all_tray_tray_icon_new (self, self->priv->_app), (self->priv->_trayIcon == NULL) ? NULL : (self->priv->_trayIcon = (g_object_unref (self->priv->_trayIcon), NULL)), _tmp1);
}


#line 72 "Process.vala"
void all_tray_process_child_died (AllTrayProcess* self) {
	GString* msg;
#line 72 "Process.vala"
	g_return_if_fail (self != NULL);
#line 73 "Process.vala"
	self->priv->_running = FALSE;
#line 74 "Process.vala"
	gtk_status_icon_set_visible (self->priv->_statusIcon, FALSE);
	msg = g_string_new ("");
#line 77 "Process.vala"
	g_string_append_printf (msg, "Child process %d (%s) died.", (gint) self->priv->_child, self->priv->_argv[0]);
#line 80 "Process.vala"
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Process, ALL_TRAY_DEBUG_LEVEL_Information, msg->str);
#line 83 "Process.vala"
	g_signal_emit_by_name (self, "process-died", self);
	(msg == NULL) ? NULL : (msg = (g_string_free (msg, TRUE), NULL));
}


gboolean all_tray_process_get_running (AllTrayProcess* self) {
	g_return_val_if_fail (self != NULL, FALSE);
#line 27 "Process.vala"
	return self->priv->_running;
}


static void all_tray_process_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AllTrayProcess * self;
	gpointer boxed;
	self = ALL_TRAY_PROCESS (object);
	switch (property_id) {
		case ALL_TRAY_PROCESS_RUNNING:
		g_value_set_boolean (value, all_tray_process_get_running (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void all_tray_process_class_init (AllTrayProcessClass * klass) {
	all_tray_process_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AllTrayProcessPrivate));
	G_OBJECT_CLASS (klass)->get_property = all_tray_process_get_property;
	G_OBJECT_CLASS (klass)->finalize = all_tray_process_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), ALL_TRAY_PROCESS_RUNNING, g_param_spec_boolean ("running", "running", "running", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("process_died", ALL_TRAY_TYPE_PROCESS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, ALL_TRAY_TYPE_PROCESS);
}


static void all_tray_process_instance_init (AllTrayProcess * self) {
	self->priv = ALL_TRAY_PROCESS_GET_PRIVATE (self);
}


static void all_tray_process_finalize (GObject* obj) {
	AllTrayProcess * self;
	self = ALL_TRAY_PROCESS (obj);
	(self->priv->_statusIcon == NULL) ? NULL : (self->priv->_statusIcon = (g_object_unref (self->priv->_statusIcon), NULL));
	self->priv->_argv = (_vala_array_free (self->priv->_argv, self->priv->_argv_length1, (GDestroyNotify) g_free), NULL);
	(self->priv->_app == NULL) ? NULL : (self->priv->_app = (g_object_unref (self->priv->_app), NULL));
	(self->priv->_trayIcon == NULL) ? NULL : (self->priv->_trayIcon = (g_object_unref (self->priv->_trayIcon), NULL));
	G_OBJECT_CLASS (all_tray_process_parent_class)->finalize (obj);
}


GType all_tray_process_get_type (void) {
	static GType all_tray_process_type_id = 0;
	if (all_tray_process_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayProcessClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_process_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayProcess), 0, (GInstanceInitFunc) all_tray_process_instance_init, NULL };
		all_tray_process_type_id = g_type_register_static (G_TYPE_OBJECT, "AllTrayProcess", &g_define_type_info, 0);
	}
	return all_tray_process_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}




