/*
 * TrayIcon.vala - Tray icon management
 * Copyright (c) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3.0 as published by the Free Software Foundation
 */

#include "TrayIcon.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <libwnck/libwnck.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include "Debug.h"




struct _AllTrayTrayIconPrivate {
	AllTrayApplication* _appForIcon;
	AllTrayProcess* _processForIcon;
	GtkStatusIcon* _statusIcon;
};

#define ALL_TRAY_TRAY_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ALL_TRAY_TYPE_TRAY_ICON, AllTrayTrayIconPrivate))
enum  {
	ALL_TRAY_TRAY_ICON_DUMMY_PROPERTY
};
static gboolean _all_tray_tray_icon_get_app_info_retry_gsource_func (gpointer self);
static void _all_tray_tray_icon_on_click_gtk_status_icon_activate (GtkStatusIcon* _sender, gpointer self);
static gpointer all_tray_tray_icon_parent_class = NULL;
static void all_tray_tray_icon_finalize (GObject* obj);



GQuark all_tray_tray_icon_error_quark (void) {
	return g_quark_from_static_string ("all_tray_tray_icon_error-quark");
}


#line 45 "TrayIcon.vala"
static gboolean _all_tray_tray_icon_get_app_info_retry_gsource_func (gpointer self) {
	return all_tray_tray_icon_get_app_info_retry (self);
}


#line 54 "TrayIcon.vala"
static void _all_tray_tray_icon_on_click_gtk_status_icon_activate (GtkStatusIcon* _sender, gpointer self) {
	all_tray_tray_icon_on_click (self);
}


#line 18 "TrayIcon.vala"
AllTrayTrayIcon* all_tray_tray_icon_construct (GType object_type, AllTrayProcess* process, AllTrayApplication* application) {
	AllTrayTrayIcon * self;
	AllTrayApplication* _tmp1;
	AllTrayApplication* _tmp0;
	AllTrayProcess* _tmp3;
	AllTrayProcess* _tmp2;
	GtkStatusIcon* _tmp4;
	GString* msg;
#line 18 "TrayIcon.vala"
	g_return_val_if_fail (process != NULL, NULL);
#line 18 "TrayIcon.vala"
	g_return_val_if_fail (application != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
#line 20 "TrayIcon.vala"
	_tmp0 = NULL;
#line 20 "TrayIcon.vala"
	self->priv->_appForIcon = (_tmp1 = (_tmp0 = application, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_appForIcon == NULL) ? NULL : (self->priv->_appForIcon = (g_object_unref (self->priv->_appForIcon), NULL)), _tmp1);
	_tmp3 = NULL;
#line 21 "TrayIcon.vala"
	_tmp2 = NULL;
#line 21 "TrayIcon.vala"
	self->priv->_processForIcon = (_tmp3 = (_tmp2 = process, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_processForIcon == NULL) ? NULL : (self->priv->_processForIcon = (g_object_unref (self->priv->_processForIcon), NULL)), _tmp3);
	_tmp4 = NULL;
#line 23 "TrayIcon.vala"
	self->priv->_statusIcon = (_tmp4 = gtk_status_icon_new (), (self->priv->_statusIcon == NULL) ? NULL : (self->priv->_statusIcon = (g_object_unref (self->priv->_statusIcon), NULL)), _tmp4);
	msg = g_string_new ("");
#line 25 "TrayIcon.vala"
	g_string_append_printf (msg, "Created tray icon for PID %d", (gint) all_tray_process_get_pid (self->priv->_processForIcon));
#line 28 "TrayIcon.vala"
	if (all_tray_application_get_wnck_app (self->priv->_appForIcon) == NULL) {
		/* The application isn't ready yet.  Wait for it to
		 become ready and _then_ get the icon setup.*/
#line 31 "TrayIcon.vala"
		gtk_status_icon_set_from_stock (self->priv->_statusIcon, GTK_STOCK_MISSING_IMAGE);
#line 32 "TrayIcon.vala"
		g_timeout_add ((guint) 50, _all_tray_tray_icon_get_app_info_retry_gsource_func, self);
	} else {
#line 34 "TrayIcon.vala"
		gtk_status_icon_set_from_pixbuf (self->priv->_statusIcon, wnck_application_get_icon (all_tray_application_get_wnck_app (self->priv->_appForIcon)));
	}
#line 37 "TrayIcon.vala"
	gtk_status_icon_set_visible (self->priv->_statusIcon, TRUE);
#line 38 "TrayIcon.vala"
	g_signal_connect_object (self->priv->_statusIcon, "activate", (GCallback) _all_tray_tray_icon_on_click_gtk_status_icon_activate, self, 0);
#line 40 "TrayIcon.vala"
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon, ALL_TRAY_DEBUG_LEVEL_Information, msg->str);
	return self;
}


#line 18 "TrayIcon.vala"
AllTrayTrayIcon* all_tray_tray_icon_new (AllTrayProcess* process, AllTrayApplication* application) {
#line 18 "TrayIcon.vala"
	return all_tray_tray_icon_construct (ALL_TRAY_TYPE_TRAY_ICON, process, application);
}


#line 45 "TrayIcon.vala"
gboolean all_tray_tray_icon_get_app_info_retry (AllTrayTrayIcon* self) {
#line 45 "TrayIcon.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 46 "TrayIcon.vala"
	if (all_tray_application_get_wnck_app (self->priv->_appForIcon) == NULL) {
#line 47 "TrayIcon.vala"
		return TRUE;
	}
#line 49 "TrayIcon.vala"
	gtk_status_icon_set_from_pixbuf (self->priv->_statusIcon, wnck_application_get_icon (all_tray_application_get_wnck_app (self->priv->_appForIcon)));
#line 50 "TrayIcon.vala"
	gtk_status_icon_set_tooltip (self->priv->_statusIcon, wnck_application_get_name (all_tray_application_get_wnck_app (self->priv->_appForIcon)));
#line 51 "TrayIcon.vala"
	return FALSE;
}


#line 54 "TrayIcon.vala"
void all_tray_tray_icon_on_click (AllTrayTrayIcon* self) {
#line 54 "TrayIcon.vala"
	g_return_if_fail (self != NULL);
#line 55 "TrayIcon.vala"
	all_tray_application_toggle_visibility (self->priv->_appForIcon);
}


static void all_tray_tray_icon_class_init (AllTrayTrayIconClass * klass) {
	all_tray_tray_icon_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AllTrayTrayIconPrivate));
	G_OBJECT_CLASS (klass)->finalize = all_tray_tray_icon_finalize;
}


static void all_tray_tray_icon_instance_init (AllTrayTrayIcon * self) {
	self->priv = ALL_TRAY_TRAY_ICON_GET_PRIVATE (self);
}


static void all_tray_tray_icon_finalize (GObject* obj) {
	AllTrayTrayIcon * self;
	self = ALL_TRAY_TRAY_ICON (obj);
	(self->priv->_appForIcon == NULL) ? NULL : (self->priv->_appForIcon = (g_object_unref (self->priv->_appForIcon), NULL));
	(self->priv->_processForIcon == NULL) ? NULL : (self->priv->_processForIcon = (g_object_unref (self->priv->_processForIcon), NULL));
	(self->priv->_statusIcon == NULL) ? NULL : (self->priv->_statusIcon = (g_object_unref (self->priv->_statusIcon), NULL));
	G_OBJECT_CLASS (all_tray_tray_icon_parent_class)->finalize (obj);
}


GType all_tray_tray_icon_get_type (void) {
	static GType all_tray_tray_icon_type_id = 0;
	if (all_tray_tray_icon_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayTrayIconClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_tray_icon_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayTrayIcon), 0, (GInstanceInitFunc) all_tray_tray_icon_instance_init, NULL };
		all_tray_tray_icon_type_id = g_type_register_static (G_TYPE_OBJECT, "AllTrayTrayIcon", &g_define_type_info, 0);
	}
	return all_tray_tray_icon_type_id;
}




